#ifndef _ENCODER_H_
#define _ENCODER_H_

#include "Link.h"
#include "verbose.h"
#include <vector>
#include "MTypes.h"
#include "Controller.h"


namespace Encoder_ns {

typedef enum {
	ENC_NO,
	ENC_DIGITAL_INCREMENTAL,
	ENC_DIGITAL_ABSOLUTE,
	ENC_DIGITAL_SEMIABSOLUTE,
	ENC_ANALOG_ABSOLUTE
} EncoderType_t;

const char * const EncoderTypeName[] = {
	"ENC_NO",
	"ENC_DIGITAL_INCREMENTAL",
	"ENC_DIGITAL_ABSOLUTE",
	"ENC_DIGITAL_SEMIABSOLUTE",
	"ENC_ANALOG_ABSOLUTE"
};

typedef enum {
        LIMIT_FLS,
        LIMIT_RLS
} EncoderLimit_t;


typedef struct {
	EncoderLimit_t startPoint;
	std::vector<double> values;
	double v0;
} SencSignature_t;

typedef enum {
	ENCSIGN_OK,
	ENCSIGN_GENERIC_ERROR,
	ENCSIGN_FEWELEMENTS_ERROR,
	ENCSIGN_DIFFSIGNS_ERROR,
	ENCSIGN_IXDIR_ERROR,
	ENCSIGN_ENC_IXDIR_ERROR
} EncSignatureError_t;

const char * const EncSignatureErrorName[] = {
	"ENCSIGN_OK",
	"ENCSIGN_GENERIC_ERROR",
	"ENCSIGN_FEWELEMENTS_ERROR",
	"ENCSIGN_DIFFSIGNS_ERROR",
	"ENCSIGN_IXDIR_ERROR",
	"ENCSIGN_ENC_IXDIR_ERROR"
};

typedef struct {
	EncSignatureError_t err;
	double abs_error;
	int enc_dir;
	int ix_dir;
} CheckSignature_t;

typedef struct {
	EncoderLimit_t startPoint;
	double SencRes;
	double SencPitch;
} SencParams_t;


class Encoder {

private:
        string                          _name;
        string                          _channel;
        Controller_ns::Controller*      _controller;

	double _epos;
	double _epos_old;
	double _evelocity;
	vector<double> _oldevelocity;

	double _ripple_on_stay;		// max variazione (ripple) ammessa quando l'encoder e' (deve essere) fermo.
	double _ripple_on_run;		// max variazione (ripple) ammessa quando l'encoder e' (deve essere) in movimento.

	double _delay_on_check;		// ritardo in sec dopo ol quale si inizia a controllare l'encoder
	int _n_delay_on_check;	// ritardo in numero di campioni (update) dopo il quale si inizia a controllare l'encoder

	double _ear;
	double _eao;

	Controller_ns::mOffOn_t _mOffOn;

	int _refwidth;			// larghezza del reference in numero di impulsi encoder

protected:
	int _verbose;
	
	EncoderType_t _etype;

	SencSignature_t _sencsign;
	SencParams_t _sencparams;

	double _precision;

        struct timeval _last_update; // succesfull update


public:
	Encoder(string name, VNOTU string channel, VNOTU Controller_ns::Controller *controller);
	~Encoder() {};

	EncoderType_t 	etype()					{ return _etype; };
	bool 		etype(EncoderType_t t, double v)	{ if (!precision(v)) return false; _etype = t; return true; };
	
	double 		precision() 				{ return _precision; };
	bool   		precision(double v) 			{ if (v<=0) return false;  _precision = v; return true; };

	bool    	update();				// acquisizione campo e ..
	bool    	eupdate(Link_ns::uvalues_t uv, double dt);	// semplicemente da uv a epos....
								// se dt == 0  aggiorna solo epos
								// se dt > 0   aggiorna anche velocita' (calcolo per ana, semplice copia per dig

        double  	epos()                  		{ return _epos; };
        bool  		epos(double v);
	// bool 		evalues(double e, double ev, double dt); ... non serve piu' gia' compreso in eupdate(.,.)

	double		ear()		{ return _ear; }; // encoder / analog ratio
	bool		ear(double v)	{ _ear = v; return true; };
	double		eao()		{ return _eao; }; // encoder / analog offset
	bool		eao(double v)	{ _eao = v; return true; };

	double		analog2enc(double a);
	double		enc2analog(double e);

	SencParams_t	SencParams();
	bool		SencParams(SencParams_t param);
	bool		SencAbsPos(double _epos0, double _epos1);
	SencSignature_t	SencSignature();
	bool		SencSignature(SencSignature_t ss);

	CheckSignature_t checkSencSignature(SencSignature_t ss);
	string		printEncSignatureError(EncSignatureError_t e){
				switch(e){
					case	ENCSIGN_OK:
					case	ENCSIGN_GENERIC_ERROR:
					case	ENCSIGN_FEWELEMENTS_ERROR:
					case	ENCSIGN_DIFFSIGNS_ERROR:
					case	ENCSIGN_IXDIR_ERROR:
					case	ENCSIGN_ENC_IXDIR_ERROR:
						return EncSignatureErrorName[e];
						break;
					default:
						return "Unknown error!";
						break;
				}
			};
	void		printSignature(){
				cout << "SencSignature:: ";
				cout << "startPoint is: ";
				switch(_sencsign.startPoint){
					case LIMIT_FLS: cout << "LIMIT_FLS, ";
					break;
					case LIMIT_RLS: cout << "LIMIT_RLS, ";
					break;
				}
				cout << "firstValue is: " << _sencsign.v0 << ", ";
				cout << "Values are: ";
                                for (std::vector<double>::iterator it = _sencsign.values.begin() ; it != _sencsign.values.end(); ++it)
                                        cout << *it << ", ";
                                cout << "|" << endl; 
			};

        bool    	defepos(double p);       // define encoder position

	double		evelocity()				{ return _evelocity; };

	double		ripple_on_stay()	{ return _ripple_on_stay; };
	bool		ripple_on_stay(double v) { _ripple_on_stay = v; return true; };

	double		delay_on_check()	{ return _delay_on_check; };
	bool		delay_on_check(double delay, double update_cycle);
	int		n_delay_on_check()	{ return _n_delay_on_check; };

	double		ripple_on_run()	{ return _ripple_on_run; };
	bool		ripple_on_run(double v) { _ripple_on_run = v; return true; };

	void            verbose(int b)        			{ _verbose = b;};
	int            	verbose()         			{ return _verbose;};

	Controller_ns::mOffOn_t	mOffOn()		{ return _mOffOn; };
	bool 		mOffOn(double m, double q)	{ _mOffOn.m = m; _mOffOn.q = q; return true; };
	bool		mOffOn(bool b)			{ _mOffOn.enabled = b; return true; };
	bool		isEnabled_mOffOn()		{ return _mOffOn.enabled; };

	int 		refwidth()			{ return _refwidth; };
	bool		refwidth(int v)			{ _refwidth = v; return true; };

}; /* class Encoder */


} /* namespace Encoder_ns */

#endif /* _ENCODER_H_ */
