#ifndef _MOTOR_H_
#define _MOTOR_H_

#include <vector>

#if 0
#include "YamsDrv.h"
#include "GalilTangoLink.h"
#endif

#include "N5Drv.h"

#include "Controller.h"
#include "MTypes.h"
#include <sys/time.h>


namespace Motor_ns {

typedef enum {
        UNKNOWN,
	DEACTIVATED,
        OFFLINE,
	MOVING,
	ACCELERATING,
	DECELERATING,
	STANDSTILL,
	BRAKED,
	ERROR,
// For PIEZO Controls
	SERVO,
	SWCTL
} MotorState_t;

const char * const MotorStateName[] = {
        "UNKNOWN",
	"DEENERGIZED", // ex DEACTIVATED
        "OFFLINE",
	"MOVING",
	"ACCELERATING",
	"DECELERATING",
	"STANDSTILL",
	"BRAKED",
	"ERROR",
	"SERVO",
	"SWCTL"
};

typedef enum {
	NOBRAKE,
	ON,
	OFF
} MotorBrake_t;

const char * const MotorBrakeName[] = {
	"NOBRAKE",
	"ON",
	"OFF"
};

typedef enum {
	NONE,
	MOTOROFF,
	MOTORBRAKED
} MotorOnStopConf_t;

const char * const MotorOnStopConfName[] = {
	"NONE",
	"MOTOROFF",
	"MOTORBRAKED"
};

class Motor {

protected:
	double 		_mspeed;
	double 		_macceleration;
	double 		_mdeceleration;
	double 		_mpos;
	MotorState_t 	_mstate;

	MotorBrake_t 	_brake;

        unsigned long 	_stop_code;
        unsigned long 	_CC_code;

private:
	string 				_name;
	string 				_channel;
	Controller_ns::Controller*	_controller;

	int _verbose;

	struct timeval _last_update; // succesfull update
	struct timeval _last_refresh;

	MotorOnStopConf_t _brake_conf;

public:
	Motor(string name, string channel, Controller_ns::Controller *controller);
	~Motor() {};

        int            	verbose();
        void           	verbose(int b);

        int            	drv_verbose();
        void           	drv_verbose(int b);

        int            	lnk_verbose();
        void           	lnk_verbose(int b);

	string		name()			{ return _name; };
        string          channel()               { return _channel; };
        void            channel(string v)       { _channel = v; };

        bool            set_mconf();
        bool            get_mconf();

        bool           	hw_write_enable()       { return _controller->hw_write_enable();};
        void           	hw_write_enable(bool b) { return _controller->hw_write_enable(b);};

	//---------------------------------------------------------------------------------
	//	GET NEW VALUES

	bool    	update();   		// update all relevant 'current' values
        bool    	refresh();      	// refresh all 'secondary' values

        bool    	refresh(double tmo);    // refresh all 'secondary' values:
                                        	// onexit:
                                        	//      true  = effective new values
                                        	//      false = time out expired! no new values

	//---------------------------------------------------------------------------------
	// 	MOTION:

	double		mspeed()		{ return _mspeed; };
	bool		mspeed(double v)	{ return _controller->speed(_channel, v); };

	double		macceleration()		{ return _macceleration; };
	bool		macceleration(double a)	{ return _controller->acceleration(_channel, a); };
	double		mdeceleration()		{ return _mdeceleration; };
	bool		mdeceleration(double a)	{ return _controller->deceleration(_channel, a); };

	double 		mpos() 			{ return _mpos; };
	bool		mpos(double p)		{ return _controller->mpos(_channel, p); };
	bool		rpos(double p)		{ return _controller->rpos(_channel, p); };

	bool		xpos(double p)		{ return _controller->xpos(_channel, p); };

	bool 		forward(double v)	{ return _controller->forward(_channel, v); }; 
	bool		backward(double v)	{ return _controller->backward(_channel, v); };

	bool		findedge(double v)	{ return _controller->findedge(_channel, v); };
	bool		findindex(double v)	{ return _controller->findindex(_channel, v); };

	bool 		stop()			{ return _controller->stop(_channel); };
	bool 		stop_no_brake()		{ return _controller->stop_no_brake(_channel); };

	bool		defmpos(double p)	{ return _controller->defmpos(_channel, p); };

	MotorState_t 	mstate() 		{ return _mstate; };
	string 		mstatus() 		{ return MotorStateName[_mstate]; };

	MotorOnStopConf_t brake_conf()		{ return _brake_conf; };
	string 		brake_conf_str()	{ return MotorOnStopConfName[_brake_conf]; };
	bool 		hw_brake_conf();

	MotorBrake_t	brake()			{ return _brake; };
	bool		brake(bool b);
	bool		motor_on()		{ return _controller->motor_on(_channel); };
	bool		motor_off()		{ return _controller->motor_off(_channel); };

        string          stop_code2str()  	{ return _controller->drv()->stop_code2str(_stop_code);};
        string          CC_code2str()      	{ return _controller->drv()->CC_code2str(_CC_code);};
	unsigned long 	stop_code()		{ return _stop_code; };
	unsigned long 	CC_code()		{ return _CC_code; };

	bool		check_motion_end()	{ return _controller->check_motion_end(_CC_code); };
	bool		check_motion_progress()	{ return _controller->check_motion_progress(_CC_code); };
	bool		check_motion_cmd()	{ return _controller->check_motion_cmd(_CC_code); };


}; /* class Motor */

} /* namespace Motor_ns */

#endif /* _MOTOR_H_ */
