#include <iostream>
#include "Motor.h"

#include <stdlib.h>
#include <string>
#include <algorithm>

namespace Motor_ns {

int     Motor::verbose() { return _verbose; }
void	Motor::verbose(int b){ 
		#ifdef _DBG_VERBOSE
                	cout << "Motor verbose(" << b << ")" << endl;
                #endif
		_verbose = b;
}

int     Motor::drv_verbose()           { return _controller->drv()->verbose(); }
void    Motor::drv_verbose(int b)      { cout << "Motor.drv_verbose()" << endl;  _controller->drv_verbose(b); }

int     Motor::lnk_verbose()           { return _controller->lnk_verbose();}
void    Motor::lnk_verbose(int b)      { cout << "Motor.lnk_verbose()" << endl; _controller->lnk_verbose(b);}


/*===============================
*
*
*
*===============================*/
Motor::Motor(string name, string channel, Controller_ns::Controller *controller) {
	std::string obj("Motor::Motor()");
	std::stringstream ss("");

	_verbose = 0;
        _name = name;

	_channel = channel;
	_controller = controller;

	#ifdef _DBG_CSTOR
		prtverbose(0, obj, "");
		cout << "Controller " << _controller << " drv ptr : " << _controller->drv() << endl;
	#else
		prtverbose(1, obj, "");
	#endif

	//_controller->drv_verbose(1);

	_mstate = UNKNOWN;

	_brake_conf = NONE;
	if (!hw_brake_conf()){
		ss.str("");
		ss << "Motor() unable to get brake conf!";
		prtverbose(0, obj, ss.str());
		throw "Motor() unable to get brake conf!";
	}
	ss.str("");
	ss << " _brake_conf = " << MotorOnStopConfName[_brake_conf];
	#ifdef _DBG_CSTOR
		prtverbose(0, obj, ss.str());
	#else
		prtverbose(1, obj, ss.str());
	#endif

	_brake = NOBRAKE;

	_last_update.tv_sec  = 0;
	_last_update.tv_usec = 0;
	_last_refresh.tv_sec  = 0;
	_last_refresh.tv_usec = 0;

	update();
	refresh();
}


/*===============================
*
Link_ns::
typedef struct {
        double motor;
        double encoder;
        double velocity;
        double analog;
        bool fls;
        bool rls;
        bool home;
        unsigned long stop_code;
        unsigned long CC_code;
        status_t status;
} uvalues_t;
typedef enum { STANDSTILL, MOVING, ERROR, OFF } status_t;

Motor_ns::
typedef enum { UNKNOWN, DEACTIVATED, OFFLINE, MOVING, ACCELERATING, DECELERATING, STANDSTILL, BRAKED, ERROR } MotorState_t;
*
*
*===============================*/
bool    Motor::update(){   // update all relevant 'current' values
	prtverbose(1, "Motor::update()", "");

	struct timeval t;
	gettimeofday(&t, NULL);
	double t_s = (double)(t.tv_sec-_last_update.tv_sec)+((double)(t.tv_usec-_last_update.tv_usec))/1000000;
	if ( t_s < _controller->updateDelay() ) return false;

	_last_update.tv_sec  = t.tv_sec;
	_last_update.tv_usec = t.tv_usec;

	Link_ns::uvalues_t uv = _controller->update(_channel);

	_mpos      = uv.motor;
	_stop_code = uv.stop_code;
	_CC_code   = uv.CC_code;

	switch(uv.status){
		case Link_ns::STANDSTILL: 	_mstate = Motor_ns::STANDSTILL; 	break;
		case Link_ns::MOVING: 		_mstate = Motor_ns::MOVING; 		break;
		case Link_ns::OFF: 		_mstate = Motor_ns::DEACTIVATED;	break;
		case Link_ns::SERVO: 		_mstate = Motor_ns::SERVO;		break;
		case Link_ns::SWCTL: 		_mstate = Motor_ns::SWCTL;		break;
		default:			_mstate = Motor_ns::UNKNOWN; 		break;
	}
	return true;
}

/*===============================
*
*  refresh all 'secondary' values:
*  onexit:
*	true  = effective new values
*	false = time out expired! no new values
*
*===============================*/
bool    Motor::refresh(double tmo){
	prtverbose(1, "Motor::refresh(tmo)", "");

	struct timeval t0, t1;
	double dt = 0;
	gettimeofday(&t0, NULL);

	double delay = _controller->updateDelay();
	if ( tmo < delay ) tmo = delay * 2;
	
	while ( !refresh() ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
	}
	return true;
}


/*===============================
*
*
Link_ns::
// values for refresh (slow)
typedef struct {
        double speed;
        double acceleration;
        double deceleration;
} rvalues_t;
*
*===============================*/
bool    Motor::refresh(){  // refresh all 'secondary' values
	prtverbose(1, "Motor::refresh()", "");

	struct timeval t;
	gettimeofday(&t, NULL);
	double t_s = (double)(t.tv_sec-_last_refresh.tv_sec)+((double)(t.tv_usec-_last_refresh.tv_usec))/1000000;
	if ( t_s < _controller->updateDelay() ) return false;
	_last_refresh.tv_sec  = t.tv_sec;
	_last_refresh.tv_usec = t.tv_usec;

	Link_ns::rvalues_t rv = _controller->refresh(_channel);

	_mspeed = rv.speed;
	_macceleration = rv.acceleration;
	_mdeceleration = rv.deceleration;

	return true;
}

/*===============================
*
*
*
*===============================*/
 bool	Motor::brake(bool b){
	string obj("Motor::brake(.)");
	std::stringstream ss;
	ss << "b= " << b;
	prtverbose(1, obj, ss.str());

	return _controller->brake(_channel, b);
}

/*===============================
*
*
*
*===============================*/
bool set_mconf(){
	return false;
}

/*===============================
*
typedef struct {
        long dbw;               // Dead Band Width = GalilMotionAccuracy positioning accuracy. Written by the client in encoder units
        double scale;           // EncoderMotorRatio = encoder impulse / motor step. Written by the client
        double pc;              // Percent of the movement to be performed from 0.0 to 1.0
        double bck;             // Backlash. Encoder units
        long dir;               // Direction of the last positioning movement in backlash compensation
        long cp;                // Type/presence of encoder: 0 = no encoder
        long nbrtr;             // Number of retries before giving up/ending with movement
} yconf_t;
*
*
*===============================*/
bool get_mconf(){
	return false;
}

/*===============================
*
*
*
*===============================*/
bool Motor::hw_brake_conf(){

	switch (_controller->hw_brake_conf(_channel)){
		case -1:
		default:
			_brake_conf = NONE;
			return false;
			break;
		case 0:
			_brake_conf = NONE;
			break;
		case 1:
			_brake_conf = MOTORBRAKED;
			break;
		case 2:
			_brake_conf = MOTOROFF;
			break;
	}
	return true;
}



} /* namespace Motor_ns */
